@@ -42,7 +42,8 @@ src_mysql <- function(dbname, host = NULL, port = 0L, username = "root",
   check_installed("RMySQL", "to connect to MySQL/MariaDB.")
   lifecycle::deprecate_warn(
     "1.0.0", "dplyr::src_mysql()",
-    details = "Please use `tbl()` directly with a database connection"
+    details = "Please use `tbl()` directly with a database connection",
+    always = TRUE
   )
 
   con <- DBI::dbConnect(
@@ -65,10 +66,13 @@ src_postgres <- function(dbname = NULL, host = NULL, port = NULL,
   check_installed("RPostgreSQL", "to connect to PostgreSQL.")
   lifecycle::deprecate_warn(
     "1.0.0", "dplyr::src_postgres()",
-    details = "Please use `tbl()` directly with a database connection"
+    details = "Please use `tbl()` directly with a database connection",
+    always = TRUE
   )
 
-  user <- user %||% if (in_travis()) "postgres" else ""
+  in_travis <- identical(Sys.getenv("TRAVIS"), "true")
+
+  user <- user %||% if (in_travis) "postgres" else ""
 
   con <- DBI::dbConnect(
     RPostgreSQL::PostgreSQL(),
@@ -95,7 +99,8 @@ src_sqlite <- function(path, create = FALSE) {
   check_dbplyr()
   lifecycle::deprecate_warn(
     "1.0.0", "dplyr::src_sqlite()",
-    details = "Please use `tbl()` directly with a database connection"
+    details = "Please use `tbl()` directly with a database connection",
+    always = TRUE
   )
 
   if (!create && !file.exists(path)) {
